@@ -3,7 +3,7 @@ typedload
 This module is the inverse of dataloader. It converts typed
 data structures to things that json can serialize.
 """
-# Copyright (C) 2018-2021 Salvo "LtWorf" Tomaselli
+# Copyright (C) 2018-2023 Salvo "LtWorf" Tomaselli
 #
 # typedload is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -48,6 +48,15 @@ class Dumper:
         When enabled, does not include fields that have the
         same value as the default in the dump.
 
+    isodates: Disabled by default.
+        Will be enabled by default from version 3.
+        When disabled, datetime.datetime, datetime.time, datetime.date
+        are dumped as lists of ints.
+
+        When enabled they are dumped as strings in ISO 8601 format.
+
+        When enabled, timezone information will work.
+
     raiseconditionerrors: Enabled by default.
         Raises exceptions when evaluating a condition from an
         handler. When disabled, the exceptions are not raised
@@ -63,12 +72,12 @@ class Dumper:
         List[
             Tuple[
                 Callable[[Any], bool],
-                Callable[['Dumper', Any], Any]
+                Callable[['Dumper', Any, Any], Any]
             ]
         ]
         The elements are: Tuple[Condition, Dumper]
         Condition(value) -> Bool
-        Dumper(dumper, value) -> simpler_value
+        Dumper(dumper, value, value_type) -> simpler_value
 
         In most cases, it is sufficient to append new elements
         at the end, to handle more types.
@@ -82,20 +91,29 @@ class Dumper:
     ones have any effect. This is to allow custom handlers to have their
     own parameters as well.
 
+    Because internal caches are used, after the first call to dump() these properties
+    should no longer be modified.
+
     There is support for:
         * Basic python types (int, str, bool, float, NoneType)
-        * NamedTuple
-        * Enum
-        * List[SomeType]
+        * NamedTuple, dataclasses, attrs, TypedDict
         * Dict[TypeA, TypeB]
-        * Tuple[TypeA, TypeB, TypeC]
-        * Set[SomeType]
+        * Enum
+        * List
+        * Tuple
+        * Set
+        * FrozenSet
+        * Path
+        * IPv4Address, IPv6Address, IPv4Network, IPv6Network, IPv4Interface, IPv6Interface
+        * datetime
     """
     def __init__(self, **kwargs) -> None:
         self.basictypes = {int, bool, float, str, NONETYPE}
 
         self.hidedefault = True
 
+        self.isodates = False
+
         # Which key is used in metadata to perform name mangling
         self.mangle_key = 'name'
 
@@ -119,7 +137,7 @@ class Dumper:
             }
 
         self.handlers = [
-            (lambda value: type(value) in self.basictypes, lambda l, value, t: value),
+            (lambda value: type(value) in self.basictypes, _identitydump),
             (lambda value: isinstance(value, tuple) and hasattr(value, '_fields') and hasattr(value, '_asdict'), _namedtupledump),
             (lambda value: '__dataclass_fields__' in dir(value), _dataclassdump),
             (lambda value: isinstance(value, (list, tuple, set, frozenset)), _iteratordump),
@@ -127,11 +145,11 @@ class Dumper:
             (lambda value: isinstance(value, Dict), lambda l, value, t: {l.dump(k): l.dump(v) for k, v in value.items()}),
             (is_attrs, _attrdump),
             (lambda value: isinstance(value, (datetime.date, datetime.time)), _datetimedump),
+            (lambda value: isinstance(value, datetime.timedelta), _timedeltadump),
             (lambda value: type(value) in self.strconstructed, lambda l, value, t: str(value)),
-
         ]  # type: List[Tuple[Callable[[Any], bool], Callable[['Dumper', Any, Any], Any]|Callable[['Dumper', Any], Any]]]
 
-        self._handlerscache = {}  # type: Dict[Type[Any], Callable[['Dumper', Any], Any]|Callable[['Dumper', Any, Any], Any]]
+        self._handlerscache = {}  # type: Dict[Type[Any], Callable[['Dumper', Any, Any], Any]]
         self._dataclasscache = {}  # type: Dict[Type[Any], Tuple[Set[str], Dict[str, Any], Dict[str, Any]]]
 
         for k, v in kwargs.items():
@@ -168,13 +186,18 @@ class Dumper:
         if func is None:
             index = self.index(value)
             f = self.handlers[index][1]
-            # It has no type parameter
-            # TODO make all handlers require it in 3.0
+            # It has no type parameter, make a lambda
             if len(signature(f).parameters) == 2:
+                import warnings
+                warnings.warn(
+                    'The type signature for the dump handlers has changed to include type hints\n'
+                    'new handlers are: f(dumper, value, annotated_type)',
+                    DeprecationWarning
+                )
                 func = lambda d, v, _: f(d, v)  # type: ignore
             else:
-                func = f
-            self._handlerscache[t] = func
+                func = f  # type: ignore
+            self._handlerscache[t] = func  # type: ignore
         return func(self, value, annotated_type)  # type: ignore
 
 
@@ -195,6 +218,15 @@ def _attrdump(d, value, t) -> Dict[str, Any]:
 
 
 def _datetimedump(d: Dumper, value: Union[datetime.time, datetime.date, datetime.datetime], t):
+    if d.isodates:
+        return value.isoformat()
+    import warnings
+    warnings.warn(
+        'Dumping datetime classes as list of values is deprecated.\n'
+            'You are encouraged to dump with isodates=True\n'
+            'This will become the default in the next major version.',
+        DeprecationWarning
+    )
     # datetime is subclass of date
     if isinstance(value, datetime.date) and not isinstance(value, datetime.datetime):
         return [value.year, value.month, value.day]
@@ -206,6 +238,10 @@ def _datetimedump(d: Dumper, value: Union[datetime.time, datetime.date, datetime
     return [value.year, value.month, value.day, value.hour, value.minute, value.second, value.microsecond]
 
 
+def _timedeltadump(d: Dumper, value: datetime.timedelta, t) -> float:
+    return value.total_seconds()
+
+
 def _namedtupledump(d: Dumper, value, t) -> Dict[str, Any]:
     field_defaults = getattr(value, '_field_defaults', {})
     # Named tuple, skip default values
@@ -236,17 +272,25 @@ def _dataclassdump(d: Dumper, value, t) -> Dict[str, Any]:
     return r
 
 def _iteratordump(d: Dumper, value: Any, t: Any) -> List[Any]:
-    itertype = getattr(t, '__args__', (Any, ))
-    if len(itertype) == 1 and (itertype[0] in d.basictypes):
-        r = []
-        iterator = iter(value)
-        try:
-            # Call one iteration with dump, to populate the cache
-            r.append(d.dump(next(iterator), itertype[0]))
-        except StopIteration:
-            return []
-        f = d._handlerscache[itertype[0]]  # type: ignore
-        r.extend((f(d, i, itertype[0]) for i in iterator))  # type: ignore
-        return r
+    itertypes = getattr(t, '__args__', (Any, ))
+    # list[T] or tuple[T, ...]
+    if (len(itertypes) == 1) or (len(itertypes) == 2 and itertypes[1] == ...):  # type: ignore
+        # This is true for lists/sets but not tuples
+        itertype = itertypes[0]
     else:
-        return [d.dump(i) for i in value]
+        itertype = Any
+
+    if itertype in d.basictypes and d.handlers[0][1] == _identitydump:
+        # Iterable of basic types, unchanged default handler for basic types
+        if isinstance(value, list):
+            # Just copy the list if it's a list
+            return value.copy()
+        else:
+            # Create a list and return it otherwise
+            return [i for i in value]
+
+    return [d.dump(i) for i in value]
+
+
+def _identitydump(d: Dumper, value: Any, t: Any) -> Any:
+    return value
