@@ -3,7 +3,7 @@ typedload
 Module to load data into typed data structures
 """
 
-# Copyright (C) 2018-2022 Salvo "LtWorf" Tomaselli
+# Copyright (C) 2018-2023 Salvo "LtWorf" Tomaselli
 #
 # typedload is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -153,8 +153,11 @@ class Loader:
     ones have any effect. This is to allow custom handlers to have their
     own parameters as well.
 
-    Using unions is complicated. If the types in the union are too
-    similar to each other, it is easy to obtain an unexpected type.
+    Because internal caches are used, after the first call to load() these properties
+    should no longer be modified.
+
+    Using unions is complicated. The best is to use tagged unions using a Literal field.
+    If the types in the union are too similar to each other, it is easy to obtain an unexpected type.
     """
 
     def __init__(self, **kwargs) -> None:
@@ -218,6 +221,7 @@ class Loader:
             (is_literal, _literalload),
             (is_typeddict, _typeddictload),
             (lambda type_: type_ in {datetime.date, datetime.time, datetime.datetime}, _datetimeload),
+            (lambda type_: type_ == datetime.timedelta, _timedeltaload),
             (lambda type_: type_ in self.strconstructed, _strconstructload),
             (is_attrs, _attrload),
             (is_any, _anyload),
@@ -736,11 +740,23 @@ def _noneload(l: Loader, value: Any, type_) -> None:
 
 def _datetimeload(l: Loader, value: Any, type_) -> Union[datetime.date, datetime.time, datetime.datetime]:
     try:
-        return type_(*value)
+        if isinstance(value, str):
+            return type_.fromisoformat(value)
+        else:
+            # This might be removed at some point in the future, but it will break data compatibility
+            return type_(*value)
     except TypeError as e:
         raise TypedloadTypeError(str(e), type_=type_, value=value)
+    except ValueError as e:
+        raise TypedloadValueError(str(e), type_=type_, value=value)
 
 
+def _timedeltaload(l: Loader, value, type_) -> datetime.timedelta:
+    try:
+        return type_(0, value)
+    except TypeError as e:
+        raise TypedloadTypeError(str(e), type_=type_, value=value)
+
 def _get_attr_converter_type(c: "Callable"):
     """
     c is a converter function passed to an attr field
